package rules

import (
	"testing"

	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
)

func TestKinds(t *testing.T) {
	cases := []struct {
		name   string
		query  string
		result string
	}{
		{
			"cm kind",
			"cm",
			"distance",
		},
	}

	ngn, err := New()
	require.NoError(t, err)
	for _, tc := range cases {
		t.Run(tc.name, func(t *testing.T) {
			result, err := ngn.Kind(tc.query)
			require.NoError(t, err)

			assert.Equal(t, tc.result, result)
		})
	}
}

func TestConverts(t *testing.T) {
	cases := []struct {
		name   string
		qty    float64
		given  string
		result float64
		target string
	}{
		{
			"c to f",
			22.0, "c",
			71.6, "f",
		},
		{
			"k to f",
			22.0, "k",
			-420.07, "f",
		},
		{
			"inches to cm",
			2.0, "inches",
			5.08, "cm",
		},
	}

	ngn, err := New()
	require.NoError(t, err)
	for _, tc := range cases {
		t.Run(tc.name, func(t *testing.T) {
			result, err := ngn.Convert(tc.qty, tc.given, tc.target)
			require.NoError(t, err)

			assert.InDelta(t, tc.result, result, 0.1)
		})
	}
}
