% vim: set ft=prolog:

unit_kind(f, temperature).
unit_kind(k, temperature).
unit_kind(c, temperature).
unit_kind(m2, area).
unit_kind(cm2, area).
unit_kind(mm, distance).
unit_kind(cm, distance).
unit_kind(m, distance).
unit_kind(km, distance).
unit_kind(feet, distance).
unit_kind(mg, mass).
unit_kind(g, mass).
unit_kind(kg, mass).
unit_kind(lb, mass).
unit_kind(cup, volume).
unit_kind(litre, volume).
unit_kind(butter, density).
unit_kind(water, density).

rule(GivenQuant, f, TargetQuant, c) :-
  TargetQuant is (GivenQuant - 32) * 5 / 9.
rule(GivenQuant, c, TargetQuant, f) :-
  TargetQuant is (GivenQuant / 5 * 9) + 32.
rule(GivenQuant, c, TargetQuant, k) :-
  TargetQuant is GivenQuant + 273.15.
rule(GivenQuant, k, TargetQuant, c) :-
  TargetQuant is GivenQuant - 273.15.

rule(GivenQuant, GivenUnit, TargetQuant, TargetUnit) :-
  rule(GivenQuant, GivenUnit, X, Y),
  rule(X, Y, TargetQuant, TargetUnit).

rat(m, km, 0.001).
rat(m, cm, 100).
rat(m, mm, 1000).
rat(feet, m, 0.3048).
rat(feet, inches, 12).

rat(g, mg, 1000).
rat(g, kg, 0.001).

ratio(X, Y, Ratio) :-
  rat(X, Y, Ratio).
ratio(X, Y, Ratio) :-
  rat(Y, X, Z), !, Ratio = 1/Z.
ratio(X, Y, Ratio) :-
  X \== Y,
  rat(Step, _, _),
  Step \== X,
  Step \== Y,
  ratio(X, Step, R1),
  ratio(Step, Y, R2),
  Ratio = R1 * R2.
ratio(X, Y, Ratio) :-
  X \== Y,
  rat(_, Step, _),
  Step \== X,
  Step \== Y,
  ratio(X, Step, R1),
  ratio(Step, Y, R2),
  Ratio = R1 * R2.

convert(GivenQuant, GivenUnit, TargetQuant, TargetUnit) :-
  unit_kind(GivenUnit, temperature),
  unit_kind(TargetUnit, temperature),
  rule(GivenQuant, GivenUnit, TargetQuant, TargetUnit).
convert(GivenQuant, GivenUnit, TargetQuant, TargetUnit) :-
  ratio(GivenUnit, TargetUnit, Ratio),
  TargetQuant is GivenQuant * Ratio.
