// Package web implements the web interface.
package web

import (
	"context"
	"fmt"
	"net/http"
	"time"

	"github.com/getkin/kin-openapi/openapi3"
	om "github.com/oapi-codegen/nethttp-middleware"

	"git.lyda.ie/kevin/units/api"
)

// Server contains the web config.
type Server struct {
	swagger *openapi3.T
	listen  string
}

// New creates a new server.
func New(listen string) (*Server, error) {
	swagger, err := api.GetSwagger()
	if err != nil {
		return nil, err
	}

	svr := &Server{
		swagger: swagger,
		listen:  listen,
	}

	return svr, nil
}

// Serve serves the web interface
func (svr *Server) Serve(ctx context.Context) {
	oapiValidator := om.OapiRequestValidator(svr.swagger)
	unitsAPI := &api.UnitsAPI{}
	server := api.NewStrictHandler(unitsAPI, []api.StrictMiddlewareFunc{})
	mux := http.NewServeMux()

	// get an `http.Handler` that we can use
	h := oapiValidator(api.HandlerFromMux(server, mux))

	s := &http.Server{
		Handler: h,
		Addr:    svr.listen,
	}

	exit := make(chan struct{})
	go func() {
		err := s.ListenAndServe()
		if err != nil {
			fmt.Println(err)
		}
		close(exit)
	}()

	select {
	case <-exit:
		fmt.Println("Server exited unexpectedly")
	case <-ctx.Done():
		shutCtx, cancel := context.WithTimeout(context.Background(), time.Duration(time.Second*30))
		defer cancel()
		s.Shutdown(shutCtx)
	}
}
